/**
  ******************************************************************************
  * @file    Watchdog_Driver.c
  * @author  Hugo Vargas Llanas, _KEMET Ltd.
  * @date    21-April-2015
  * @version     v1.1
  * @brief   This file contains all the functions involving the watchdog Timer
  ******************************************************************************
  */


/* Includes ------------------------------------------------------------------*/
#include "Watchdog_Driver.h"

/* Private Variables -----------------------------------------------------*/
static  Watchdog_Config_Type Config          = {0};            // Configuration Structure
static  uint8_t              Config_Set_Flag = 0;              // Flag indicating that module has been configured

/** @defgroup Watchdog Watchdog driver
  * @ingroup IRPyro_microcontroller_interface
  * @{
  */
/**
  * @brief  Initializes the Watchdog Timer used incase the MCU hangs up
  */
void Watchdog_Init (Watchdog_Config_Type * External_Config)
{

    // Check Configuration
    if(External_Config)
    {
        Config.Enabled                                      =   External_Config->Enabled;
        Config.Short_Timeout                            =       External_Config->Short_Timeout;
        Config.Long_Timeout                             =       External_Config->Long_Timeout;
        Config.Timeout_Setting_Default      =       External_Config->Timeout_Setting_Default;

        // Only initialize if Watchdog Enabled Flag is set
        if(Config.Enabled)
        {
            // Enable Writing
            IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);

            // Set prescaler to 16
            IWDG_SetPrescaler(IWDG_Prescaler_256);

            // Set reload value
            if(Config.Timeout_Setting_Default == Long_Timeout)
            {
                IWDG_SetReload(Config.Long_Timeout);
            }
            else
            {
                IWDG_SetReload(Config.Short_Timeout);
            }

            // Disable Writing
            IWDG_WriteAccessCmd(IWDG_WriteAccess_Disable);

            // Reload Counter
            IWDG_ReloadCounter();

            // Initialize Watchdog
            IWDG_Enable();
        }

        // Set configure flag
        Config_Set_Flag=1;
    }

}

/**
  * @brief  Used to increase the timeout period.
  * This is used when the device is going to be doing a very time consuming task
  * that would temporarily freeze the main loop.
  */
void Timeout_Set(Timeout_Length_Type Timeout_Length_Types)
{

    // Only change if configure flag is set
    if(Config_Set_Flag)
    {

        // Enable Writing
        IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);

        // Set reload value
        if(Timeout_Length_Types == Long_Timeout)
        {
            IWDG_SetReload(Config.Long_Timeout);
        }
        else
        {
            IWDG_SetReload(Config.Short_Timeout);
        }

        // Disable Writing
        IWDG_WriteAccessCmd(IWDG_WriteAccess_Disable);

        // Ensure Watchdog is ready
        while(IWDG_GetFlagStatus(IWDG_FLAG_RVU)) {};

        // Reload Counter
        IWDG_ReloadCounter();
    }
}
/**
  * @brief  Reloads the counter for the watchdog to avoid resetting when the hardware has not hung up.
  */
void Reload_Counter (void)
{

    // Only set if configure flag is set
    if(Config_Set_Flag)
    {
        // Reload Counter
        IWDG_ReloadCounter();
    }
}
/** end of Watchdog driver group
  * @}
  */

/* ********** Copyright (c) 2018 _KEMET, Ltd. **********END OF FILE************/
