/**
******************************************************************************
* @file    message_unpack.c
* @author  _KEMET, Ltd
* @date    March 2018
* @version Release 1.0.6
* @copyright (c) 2018 _KEMET, Ltd
* @brief   IRPyro evaluation tool with API integration.
*
* @verbatim
 ===============================================================================
                      ##### Description #####
 ===============================================================================
  [..]
  From a stream of command bytes coming from a GUI the byte indicating a command
  is found and broken down in it main components: wild card (sensor/ sensors to
  be applied), payload and crc.
  [..]
 @endverbatim
  ******************************************************************************
   @attention <h2><center>&copy; COPYRIGHT 2018 _KEMET, Ltd</center></h2>
   @verbatim
    Copyright (c) 2018, _KEMET, Ltd
    All rights reserved.

    THIS SOFTWARE IS PROVIDED BY _KEMET, Ltd ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL _KEMET, Ltd BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   @endverbatim

  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
// #define USE_TEST_BUFFER
#ifndef MESSAGE_UNPACK
#include "message_unpack.h"
#endif

/** @defgroup Message_Handler Message Handler
  * @ingroup eevt_main
  * @brief  Packs and unpacks messages to and from the GUI
  * @verbatim
  *
  * @endverbatim
  * @note
  * @{
  */
/** @defgroup message_unpack Message Unpack
  * @ingroup Message_Handler
  * @brief Process a stream of bytes to find a valid GUI command
  * @verbatim
  *
  * @endverbatim
  * @note
  * @{
  */
/** @typedef decode_state_type
*/
typedef enum
{
    initialize,
    get_payload,
    error_report,
    complete
} decode_state_type;
/**
 * @struct command_table
 * @verbatim
 * Individual commands have the following structure:
 *  <GUI command> ::= <cmd code> <xor validation byte>
 *  <GUI command> ::= <cmd code> <sensor address> <xor validation byte>
 *  <GUI command> ::= <cmd code> <sensor address> <payload> <xor validation byte>
 * @endverbatim
*/
struct command_table
{
    uint8_t cmd_code;     // unique value
    uint8_t cmd_wildcard; // 0: no wild card. 1: sensor number or all sensors
    uint8_t cmd_payload;  // maximum number of bytes for expected payload.
};
/** @struct command_table
 * Array with the commands recognized by the message unpack
*/
static const struct command_table IRPyro_cmd[CMD_LIST_SIZE] =
{
    {0x00,0x00,0x00}, // null test command
    {0x01,0x00,0x00}, // send firmware version
    {0x02,0x00,0x00}, // read asic version
    {0x03,0x01,0x00}, // CCP read
    {0x04,0x01,0x05}, // CCP write
    {0x05,0x01,0x00}, // AFE read
    {0x06,0x01,0x04}, // AFE write
    {0x07,0x00,0x00}, // WUP read
    {0x08,0x01,0x06}, // WUP write
    {0x09,0x00,0x00}, // start streaming
    {0x0A,0x00,0x00}, // dummy
    {0x0B,0x00,0x00}, // dummy
    {0x0C,0x00,0x00}, // start status
    {0x0D,0x00,0x00}, // stop status
    {0x0E,0x00,0x00}, // goto sleep - all sensors
    {0x0F,0x00,0x00}, // wake up - all sensors
    {0x10,0x00,0x00}, // reset soft - sensor
    {0x11,0x00,0x00}, // reset hard - sensor
    {0x12,0x00,0x00}, // reset hardware - board
    {0x13,0x00,0x00}, // sampling rate read
    {0x14,0x00,0x01}, // sampling rate write
    {0x15,0x00,0x00}, // logical channels read
    {0x16,0x00,0x07}, // logical channels write
    {0x17,0x00,0x02}, // set emitter timings
    {0x18,0x00,0x01}, // set emitter state
    {0x19,0x00,0x00}, // get failure flags
    {0x1A,0x00,0x00}, // board type read
    {0x1B,0x00,0x00}, // unit id read
    {0x1C,0x00,0x00}, // unit id write
    {0x1D,0x00,0x00}, // second unit id read
    {0x1E,0x00,0x00}, // get dark pixel state
    {0x1F,0x00,0x00}, // set dark pixel state
    {0x77,0x00,0x00}, // stop streaming
    {0x21,0x00,0x00}, // report error
    {0x22,0x00,0x00}  // dummy
};
#ifdef USE_TEST_BUFFER
/* ----------------- BEGIN OF TEST DATA  ----------------- */
static uint8_t byte_stream_index = 0; // for test only replace with getchar method
static const uint8_t byte_stream[] =
{
    0x01, 0x01, 0x01
};

/* ----------------- END OF TEST DATA  ----------------- */
#endif

#define CMD_BUFFER_SIZE 20
#define PAYLOAD_SIZE     7

uint8_t cmd_completed, cmd_error, cmd_empty; // flags for the state machine
uint8_t cmd_input;                           // seed for the command byte stream
uint8_t cmd_search_result;                   // successfully decoded command code
uint8_t cmd_buffer[CMD_BUFFER_SIZE] = {0};   // command byte segment to process
uint8_t payload_array[PAYLOAD_SIZE] = {0};   // payload of the decoded command
uint8_t get_char_result;
uint8_t idx_buffer;
uint8_t max_reads;
uint8_t crc_calc;
uint8_t continue_process_stream;
decode_result_type op_result;
decode_state_type next_state; // control for the state machine

/**
* @brief gets a byte from the UART or the byte array
*/
static uint8_t byte_get(uint8_t *output_byte)
{
#ifndef USE_TEST_BUFFER
    get_char_result = message_getchar(output_byte);
#endif
#ifdef USE_TEST_BUFFER
    static uint8_t byte_stream_index;
    get_char_result = 1;
    *output_byte = byte_stream[byte_stream_index]; // for test only replace with getchar method
    if (byte_stream_index < sizeof(byte_stream))
        ++byte_stream_index; // get the next byte in the test buffer
    else
        byte_stream_index=0;
#endif
    return get_char_result;
}
/**
* @brief on valid crc assigns cmd_code
*/
static void assign_cmd_code(void)
{
    if (cmd_buffer[0] == 0x77)
    {
        cmd_search_result = 0x20;
    }
    else
    {
        cmd_search_result = cmd_buffer[0];
    }
    continue_process_stream = 0;
    next_state     = get_payload;
}
/**
* @brief gets the next byte to process
*/
static void process_next_byte(void)
{
    ++idx_buffer;
    cmd_buffer[idx_buffer] = cmd_input;
    if (crc_calc == cmd_input)
    {
        assign_cmd_code();
    }
    else
    {
        -- max_reads;
    }
}
/**
* @brief process stream until crc checks or fails
*/
static void process_stream(void)
{
    uint8_t retries;
    retries = 3;
    crc_calc = crc_calc ^ cmd_input;
    do
    {
        get_char_result= byte_get(&cmd_input);
        if (get_char_result==0)
        {
            Micro_controller_delay_ms(10);
        }
        -- retries;
    }
    while (get_char_result == 0 && retries);

    if (get_char_result)
    {
        process_next_byte();
    }
    else
    {
        continue_process_stream = 0;
        next_state = error_report;
    }
}
/**
* @brief looks for a command in the byte stream
*/
static void search_for_command(void)
{
    memset(cmd_buffer, 0x00, CMD_BUFFER_SIZE);
    cmd_buffer[idx_buffer] = cmd_input;
    do
    {
        process_stream();
    }
    while(continue_process_stream  && max_reads);
}
/**
* @brief On success returns a buffer with command and payload (if any)
* @return cmd_search_result code for the command currently in process
* @return cmd_buffer        segment of the current byte stream with command and payload
* @return next_state        index for the state machine
*/
static void fn_decode_initialize(void)
{
    cmd_completed = 0;
    cmd_empty     = 0;
    cmd_error     = 0;
    get_char_result = 0;
    idx_buffer = 0;
    max_reads = 10;
    crc_calc = 0xff; // seed crc calculation
    continue_process_stream = 1;

    get_char_result= byte_get(&cmd_input);
    if (get_char_result)
    {
        search_for_command();
    }
    else
    {
// no command to process, do nothing
        cmd_empty  = 1;
        next_state = complete;
    }
}
/**
* @brief gets the payload from a command buffer array coming from the GUI
* @param[out] payload  destination of cmd_buffer payload
* @param[in] cmd_buffer source byte array being decoded
* @param[in] cmd_tab_idx recovers the payload size from the look
*/
static void payload_get( uint8_t *payload, uint8_t const *cmd_buffer, uint8_t cmd_tab_idx)
{
    uint8_t payload_cnt = IRPyro_cmd[cmd_tab_idx].cmd_payload;
    if (IRPyro_cmd[cmd_tab_idx].cmd_wildcard)
    {
        // +2 skips the command code plus the wild card
        memmove(payload, cmd_buffer+2, payload_cnt);
    }
    else
    {
        // +1 skips the command code
        memmove(payload, cmd_buffer+1, payload_cnt);
    }
}
/**
* @brief fills the payload buffer if the command requires it
* @return payload           payload buffer
* @return next_state        index for the state machine
*/
static void fn_decode_payload_get(void)
{
    if (IRPyro_cmd[cmd_search_result].cmd_payload)
    {
        payload_get(payload_array, cmd_buffer, cmd_search_result);
    }
    next_state = complete;
}
/**
* @brief sets the error flag
* @return cmd_error         error flag
* @return next_state        index for the state machine
*/
static void fn_decode_error_report(void)
{
    cmd_error = 1;
    next_state = complete;
}
/**
* @brief sets the completion flag and the next state
* @return cmd_completed state machine end flag
* @return next_state           index for the state machine
*/
static void fn_decode_complete(void)
{
    cmd_completed = 1;
    next_state = initialize;
}
/**
* @brief gives access to the command code and the corresponding payload (if any)
* @return cmd_code_result  command processed successfully
* @return cmd_payload      corresponding payload (if any)
*/
void message_unpack_result(uint8_t *cmd_code_result, uint8_t *sensor_destination, uint8_t *cmd_payload)
{
    *cmd_code_result    = cmd_search_result;
    *sensor_destination = cmd_buffer[1];
    if (IRPyro_cmd[cmd_search_result].cmd_payload)
    {
        memcpy(cmd_payload, payload_array, PAYLOAD_SIZE);
    }
}
/**
* @brief evaluates the result
*/
static void decode_result_assign(void)
{
    op_result = decode_success;
    if (cmd_empty)
    {
        op_result = no_data;
    }
    if (cmd_error)
    {
        op_result = decode_fail;
    }
}
/**
 * @brief FSM to decode a received command
 * If the command is valid, the command code and the payload (if any)
 * are made available for the caller function.
*/
decode_result_type message_unpack_decode(void)
{
    static void (* const pf[5])(void) =
    {
        fn_decode_initialize,
        fn_decode_payload_get,
        fn_decode_error_report,
        fn_decode_complete
    };
    next_state = initialize;
    do
    {
        (*pf[next_state])();
    }
    while (!cmd_completed && !cmd_error && !cmd_empty);
    decode_result_assign();
    return op_result;
}
/** end of message unpack group
  * @}
  */
/** end of message_handler group
  * @}
  */
/* ********** Copyright (c) 2018 _KEMET, Ltd. **********END OF FILE************/
