/**
******************************************************************************
* @file    Micro_controller_template.c
* @author  _KEMET, Ltd
* @date    March 2018
* @version Release 1.0.6
* @copyright (c) 2018 _KEMET, Ltd
* @brief   Micro controller specific code to access the IRPyro sensor.
* @verbatim
===============================================================================
##### How to use this template #####
===============================================================================
[..]
(#) The Micro_controller_template.c and Micro_controller_template.h contain
    the minimal declarations and function implementation that are specific
    to the hardware configuration in which an IRPyro sensor is connected.
(#) This template is organized in two layers:
    (++) Private functions which are micro controller and board specific,
         for initialization and configuration.
         (+++) GPIO initialization
               GPIO_structure_init(): Enables the specific port-pin combination
                                      specific to ARM micro controller
              (see ../Libraries/STM32F30x_StdPeriph_Driver/src/stm32f30x_gpio.c)
         (+++) I2C initialization
               Micro_controller_I2C_init(): High level load of the data structure
                                     specific to I2C implementation on ARM micro controllers.
               i2c_Init(): Detailed initialization of I2C specific to ARM micro controllers.
         (+++) Timer configuration
         (+++) External Interrupt configuration
         (+++) USART configuration
               Locate the pins dedicated to the USART in your micro controller
               Change the values accordingly on Micro_controller_UART_Init()
               Configure RealTerm or similar with this values:
                  Baud rate: 921600
                  Parity   : None
                  Data Bits: 8
                  Stop Bits: 1
                  Hardware Flow Control: None
               Capture streaming data.
    (++) Public interface functions to the IRPyro API. The purpose is to provide
         a layer to hide the specifics of the micro controller regarding:
         (+++) Micro_controller_template functions
               (-) Micro_controller_template_init(): Groups together the private
                                                     configuration functions.
               (-) Micro_controller_delay_ms(): To provide a delay function,
                                                required only at power-up of the IRPyro sensor
         (+++) GPIO Control Functions
               (-) INT_pin_read(): If this pin is active low FIFO data is ready.
               (-) CS_pin_set()  : Control of IRPyro power line.
               (-) power_set()   : Power up toggle sequence.
         (+++) I2C Control Functions prototypes
               (-) i2c_Write()   : Sends commands to the IRPyro via I2C
               (-) i2c_Read()    : Retrieves status / data from the IRPyro via I2C
         (+++) USART Access Functions prototypes
               (-) UART_byte_get()   : Recovers a byte from the USART
               (-) UART_stream_send(): Sends collected data from the IRPyro
(#) The present example is tailored for the STM32F303 NUCLEO-32 micro controllers
* @endverbatim
******************************************************************************
   @attention <h2><center>&copy; COPYRIGHT 2018 _KEMET, Ltd</center></h2>
   @verbatim
    Copyright (c) 2018, _KEMET, Ltd
    All rights reserved.
    THIS SOFTWARE IS PROVIDED BY _KEMET, Ltd ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL _KEMET, Ltd BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   @endverbatim
******************************************************************************
*/
/* Includes ------------------------------------------------------------------*/
#include "Micro_controller_template.h"
/** @defgroup IRPyro_microcontroller_interface IRPyro micro controller interface
 * @brief  Provides access to: GPIO, I2C, UART, LEDs, TIM for the NUCLEO-F303K8 board
 * @verbatim
 * @endverbatim
 * @attention Modify accordingly depending on architecture
 * @{
*/
/* Private variables ---------------------------------------------------------*/
/** @defgroup IRPyro_I2C IRPyro I2C
 * @brief Intermediate level functions to access I2C
 * @verbatim
 *    Module provides initialization, read and write functions that allow
 *    architecture independence to the IRPyro API
 * @endverbatim
 * @note Modify accordingly depending on architecture
 * @{
 */
/** @defgroup I2C_G1_IRPyro IRPyro I2C Suggested Configuration Values
 * @brief I2C clock and timeout values for ARM
 * @verbatim
 *    Sets the clock and timeout constants for the I2C read and write
 *    functions
 * @endverbatim
 * @note Modify accordingly depending on architecture
 * @{
 */
#define I2C_CLK_TIMING_DEFAULT 0x00A00D26    // Refer to STM AN4235 --> 0x00800D22 0x00A00D26
#define I2C_TIMEOUT_DEFAULT    50            // Number of tries for a flag (5000)
#define I2C_SCL_PIN            GPIO_Pin_7    // SCL pin
#define I2C_SDA_PIN            GPIO_Pin_6    // SDA pin
#define I2C_SCL_SRC            GPIO_PinSource7    // SCL pin
#define I2C_SDA_SRC            GPIO_PinSource6    // SDA pin
#define I2C_PORT               GPIOB         // PORT B
/** end of IRPyro I2C Suggested Configuration Values
 * @}
 */
/** @defgroup I2C_G2_IRPyro IRPyro I2C Configuration Structure
 * @brief    Data structure to group the I2C configuration values
 * @verbatim
 *    This structure is accessed by i2c_Write and i2c_Read
 * @endverbatim
 * @note Modify accordingly depending on architecture
 * @{
 */
/**
 * @struct I2C_Config Micro_controller_template.c
 * @brief  IRPyro Analog Front End (AFE) register structure definition
 */
typedef struct I2C_Config
{
    uint32_t  CLK_Timing;  ///< clock timing
    uint32_t  Timeout; ///< timeout in counts
    uint16_t  SCL_Pin; ///< I2C clock pin
    uint16_t  SDA_Pin; ///< I2C data pin
    uint8_t   SCL_Src; ///< I2C clock source pin
    uint8_t   SDA_Src; ///< I2C data source pin
    GPIO_TypeDef* GPIOx; ///< I2C Port
} I2C_Config_Type;
typedef enum {Error = 0, Success = !Error } Status;
/** end of IRPyro I2C Configuration Structure
 * @}
 */
/** end of IRPyro access to I2C functions
 * @}
 */
/* Private functions ---------------------------------------------------------*/
static void Micro_controller_GPIO_init(void);
static void Micro_controller_I2C_init(void);
static void Micro_controller_UART_init(void);
static void Micro_controller_Systick_init(uint16_t frequency);
static void Micro_controller_External_interrupt_init(void);
/* GPIO Control Functions prototypes ******************************************/
static void GPIO_structure_init(uint32_t GPIO_Pin_x, GPIOMode_TypeDef GPIO_Mode,
                                GPIOOType_TypeDef Output_Type, GPIOSpeed_TypeDef Speed,
                                GPIOPuPd_TypeDef PuPd, GPIO_TypeDef* GPIOx);
/* I2C Initialization function ************************************************/
static void i2c_Init (I2C_Config_Type* Config);
/* External Interrupt private function prototypes -----------------------------------------------*/
static void EXTI9_5_Config(void);
static void EXTI4_Config(void);
/* LED configuration of Backplane boards function prototype -----------------------------------------------*/
static void Micro_controller_LED_configure(void);
static void Micro_controller_TIM_init(void);
/** @defgroup IRPyro_Micro_controller Micro controller interface
 * @brief Micro controller architecture dependent functions
 * @verbatim
 *    Functions to initialize ports, timers and delays
 *    implemented for the STM32 micro controllers
 * @endverbatim
 * @note Modify accordingly depending on architecture
 * @{
 */
/**
 * @brief  Groups all required initialization in one function
 * @attention Modify accordingly depending on architecture
 */
void Micro_controller_template_init(void)
{
    NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);
    Micro_controller_GPIO_init();
    Micro_controller_I2C_init();
    Micro_controller_UART_init();
    Micro_controller_Systick_init(1000);
    Micro_controller_External_interrupt_init();
    Micro_controller_LED_configure();
    Micro_controller_TIM_init();
}
/**
 * @brief  Example of board initialization to set up all the ports and
 *         peripherals. (Read_Me.txt)
 *
 * @attention Modify this function to enable the appropriate pins location for
 *            INT and CS lines where the IRPyro is connected.
 */
void Micro_controller_GPIO_init(void)
{
    // IRPyro INT lines on the IRPyro Base Board (dIRBB01) and demo boards
    // set pins 4, 5 , 6 & 7 of port A as normal INPUTS
    GPIO_structure_init(GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_11 | GPIO_Pin_12,
                        GPIO_Mode_IN,
                        GPIO_OType_PP,
                        GPIO_Speed_2MHz,
                        GPIO_PuPd_UP,
                        GPIOA );
    // IRPyro CS lines lines on the IRPyro Base Board (dIRBB01) and demo boards
    // set pins 1, 2, 3, 4 & 5 of port B as normal OUTPUTS
    GPIO_structure_init(GPIO_Pin_1 |GPIO_Pin_2 | GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5,
                        GPIO_Mode_OUT,
                        GPIO_OType_PP,
                        GPIO_Speed_2MHz,
                        GPIO_PuPd_NOPULL,
                        GPIOB );
    // IRPyro I2C Lines on the IRPyro Base Board (dIRBB01) and demo boards
    // set pins 6 & 7 of port B as in ALTERNATE FUNCTION
    GPIO_structure_init(GPIO_Pin_6 | GPIO_Pin_7,
                        GPIO_Mode_AF,
                        GPIO_OType_OD,
                        GPIO_Speed_50MHz,
                        GPIO_PuPd_NOPULL,
                        GPIOB );
}
/**
 * @brief     Calls I2C initialization. See Read_Me.txt
 * @attention Modify accordingly depending on the initialization of the I2C lines
 *            on the micro controller architecture.
 */
void Micro_controller_I2C_init(void)
{
    I2C_Config_Type         I2C_Config;
    I2C_Config.CLK_Timing = I2C_CLK_TIMING_DEFAULT;
    I2C_Config.Timeout  = I2C_TIMEOUT_DEFAULT;
    I2C_Config.SCL_Pin  = I2C_SCL_PIN;
    I2C_Config.SDA_Pin  = I2C_SDA_PIN;
    I2C_Config.SCL_Src  = I2C_SCL_SRC;
    I2C_Config.SDA_Src  = I2C_SDA_SRC;
    I2C_Config.GPIOx    = I2C_PORT;
    i2c_Init(&I2C_Config);
}
/**
  * @brief  Initializes the USART 2 on the STM32F303K8
  *         PORT A and pins 2,15 are used.
  *         Refer to STM document RM0316 section 29 and the stm32f30x_usart.c
  */
void Micro_controller_External_interrupt_init(void)
{
    EXTI4_Config();
    EXTI9_5_Config();
}
/**
  * @brief  Initializes the USART 2 on the STM32F303K8
  *         PORT A and pins 2,15 are used.
  *         Refer to STM document RM0316 section 29 and the stm32f30x_usart.c
  */
void Micro_controller_UART_init(void)
{
    // CONFIGURATIONS
    GPIO_InitTypeDef       GPIO_InitState;                            // Port and pin
    USART_InitTypeDef      USART_InitStruct ;                         // USART
    USART_ClockInitTypeDef USART_InitClockStruct;                     // USART clock
    NVIC_InitTypeDef   NVIC_InitStructure ;                          // USART interrupt
    USART2->ICR    = 0xE;
    RCC->CFGR3    |= 0x10;
    RCC->CFGR      = 0x0034000A;
    SYSCFG->CFGR1 |= (uint32_t)0x100000;
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2,ENABLE);             // enables clocks for USART2
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);               // enables pins for USART2 on Port A

    GPIO_InitState.GPIO_Pin   = GPIO_Pin_2 | GPIO_Pin_15 ;            // Pin configuration for port A
    GPIO_InitState.GPIO_Mode  = GPIO_Mode_AF ;
    GPIO_InitState.GPIO_Speed = GPIO_Speed_10MHz ;
    GPIO_InitState.GPIO_OType = GPIO_OType_PP ;
    GPIO_InitState.GPIO_PuPd  = GPIO_PuPd_NOPULL ;
    GPIO_Init(GPIOA,&GPIO_InitState) ;
    GPIO_PinAFConfig(GPIOA,GPIO_PinSource15,GPIO_AF_7);               // Pin source enabled
    GPIO_PinAFConfig(GPIOA,GPIO_PinSource2 ,GPIO_AF_7);

    USART_DeInit(USART2);                                             // USART2 initialization
    USART_StructInit(&USART_InitStruct);
    USART_ClockStructInit(&USART_InitClockStruct);
    USART_InitStruct.USART_BaudRate = 921600;                         // see Micro_controller_CLOCK_init
    USART_ClockInit(USART2,&USART_InitClockStruct);
    USART_Init(USART2,&USART_InitStruct) ;                            // Usart 2 initialization
    USART_ITConfig(USART2,USART_IT_RXNE,ENABLE) ;


    NVIC_InitStructure.NVIC_IRQChannel                   = USART2_IRQn;  // we want to configure the USARTX interrupts
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;    // USARTX priority group
    NVIC_InitStructure.NVIC_IRQChannelSubPriority        = 0;    // sub priority inside the group
    NVIC_InitStructure.NVIC_IRQChannelCmd                = ENABLE ;      // the USARTX interrupts are globally enabled
    NVIC_Init(&NVIC_InitStructure);                                      // the properties are passed to the NVIC_Init function which takes care of the low level stuff
    NVIC_EnableIRQ(USART2_IRQn);
    USART_Cmd(USART2,ENABLE) ;
}
/**
 * @brief  Example of board timer initialization to set a delay
 *
 *         The IRPyro sensor requires a delay loop to allow settling
 *         Refer to section "12.8 Device Power Up and Reset" in the Reference Manual
 *         This can be achieved using an interrupt based counter as shown in this example.
 *
 * @attention This function is required to use Micro_controller_delay_ms()
 */
void Micro_controller_Systick_init(uint16_t frequency)
{
    RCC_ClocksTypeDef RCC_Clocks;
    RCC_GetClocksFreq(&RCC_Clocks);
    (void) SysTick_Config(RCC_Clocks.HCLK_Frequency / frequency);
}
static volatile uint32_t ticks;
static volatile uint32_t ticks_down;
/**
 * @brief  Updates the both tick counters on a Systick interrupt every millisecond
 * @attention Modify accordingly depending on architecture
 */
void SysTick_Handler(void)
{
    if (ticks < UINT32_MAX )
    {
        ++ticks;
    }
    if (ticks_down)
    {
        --ticks_down;
    }
}
/**
 * @brief  Sets the tick_down to monitor a time out
 * @attention Modify accordingly depending on architecture
 */
void Micro_controller_timeout_set(uint32_t timeout)
{
    ticks_down = timeout;
}
/**
 * @brief  Recovers the tick_down value to check is not expired
 * @attention Modify accordingly depending on architecture
 */
uint32_t Micro_controller_timeout_get(void)
{
    return ticks_down;
}
/**
 * @brief  Sets the tick counter
 * @param delay integer value in milliseconds
 * @attention Modify accordingly depending on architecture
 */
void Micro_controller_delay_set(uint32_t delay)
{
    ticks = delay;
}
/**
 * @brief  Retrieves the tick counter
 * @attention Modify accordingly depending on architecture
 */
uint32_t Micro_controller_delay_get(void)
{
    return ticks;
}
/**
 * @brief  return the system clock as milliseconds
 * @attention Modify accordingly depending on architecture
 */
static uint32_t SysTick_millis(void)
{
    return ticks;
}
/**
 * @brief  Example of delay function in micro seconds.
 *         Refer to section "12.8 Device Power Up and Reset" in the Reference Manual
 * @param time_us integer value in micro seconds
 * @attention Modify accordingly depending on architecture
 */
void Micro_controller_delay_us(uint32_t time_us)
{
#define CLOCK_CYCLES_PER_INSTRUCTION    1
#define CLOCK_FREQ                      72  //IN MHZ (e.g. 16 for 16 MHZ)
    for (int i=0; i<time_us; ++i)
    {
        for (int cycleCount = CLOCK_FREQ / CLOCK_CYCLES_PER_INSTRUCTION; cycleCount>0; --cycleCount)
        {
        }
    }
}
/**
 * @brief  Example of delay function.
 *
 *         This delay function is required to allow the IRPyro
 *         sensor to stabilize on power up. In this example
 *         SysTick_Handler() provides the delay.
 *         Refer to section "12.8 Device Power Up and Reset" in the Reference Manual
 *
 * @param  time_ms delay time in milliseconds.
 * @retval None.
 * @attention Modify accordingly depending on architecture
 */
void Micro_controller_delay_ms(uint32_t time_ms)
{
    uint32_t start, end;
    start = SysTick_millis();
    end = start + time_ms;
    if (start < end)
    {
        while ((SysTick_millis() >= start) && (SysTick_millis() < end))
        {
            // do nothing
        }
    }
    else
    {
        while ((SysTick_millis() >= start) || (SysTick_millis() < end))
        {
            // do nothing
        };
    }
}
/** end of Micro_controller_template
 * @}
 */
/**@defgroup IRPyro_GPIO IRPyro GPIO
 * @brief  access to General I/O functions (GPIO)
 * @verbatim
 *    Provides the data structure.
 *    For ARM type micro controllers enable clocks and configure pins.
 * @endverbatim
 * @attention Modify accordingly depending on architecture
 * @{
 */
/** @defgroup GPIO_F1_IRPyro IRPyro initialization of GPIO structure.
 * @brief    Initializes the GPIO structure
 * @verbatim
 *    The GPIO implementation on the STM32F3 micro controllers
 *    provides architecture independence to the IRPyro API
 * @endverbatim
 * @attention Modify accordingly depending on architecture
 * @{
 */
/**
 * @brief  Populates and initializes GPIO structures.
 * @param  GPIO_Pin_x:  Pin Number (can use bitwise operation for multiple pins)
 * @param  GPIO_Mode:   Port Mode, Input, Output, Analogue, Alternate Function
 * @param  Output_Type: PP or OD
 * @param  Speed:       GPIO Speed
 * @param  PuPd:        Pull up configuration, Up, Down, None
 * @param  GPIOx:       GPIO Port: A,B,C,D,E,F
 * @retval None
 * @verbatim This function uses the ST libraries to configure the registers of
 *           the GPIO pins. The purpose of this function is to offer a higher
 *           level configuration for the pins, ensuring that the structure is
 *           fully populated every time.
 * @endverbatim
 * @attention Modify this function so the pins where the IRPyro are connected can be
 *            accessible according to the requirements of the chosen MCU
 */
void GPIO_structure_init(uint32_t GPIO_Pin_x,
                         GPIOMode_TypeDef GPIO_Mode,
                         GPIOOType_TypeDef Output_Type,
                         GPIOSpeed_TypeDef Speed,
                         GPIOPuPd_TypeDef PuPd,
                         GPIO_TypeDef* GPIOx)
{
    // Declare Structure
    GPIO_InitTypeDef my_GPIO_Struc;
    // Enable respective GPIO clock
    if (GPIOx==GPIOA)
    {
        RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);
    }
    if (GPIOx==GPIOB)
    {
        RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOB, ENABLE);
    }
    // Populate Structure
    my_GPIO_Struc.GPIO_Pin        = GPIO_Pin_x;
    my_GPIO_Struc.GPIO_Mode       = GPIO_Mode;
    my_GPIO_Struc.GPIO_OType      = Output_Type;
    my_GPIO_Struc.GPIO_Speed      = Speed;
    my_GPIO_Struc.GPIO_PuPd       = PuPd;
    // Initialize Port
    GPIO_Init(GPIOx, &my_GPIO_Struc);
}
/** end of IRPyro initialization of GPIO structure.
 * @}
 */
/** @defgroup GPIO_F2_IRPyro IRPyro GPIO control functions.
 * @brief    Read and write functions for the CS and INT
 * @verbatim
 *    For the IRPyro sensor the CS pin acts like a chip enable powering up the
 *    sensor.
 *    The INT pin on the IRPyro, when enabled, indicates that data is ready on
 *    the FIFO, for collection
 * @endverbatim
 * @attention Modify this function so affected pins of the MCU can work as an INPUT
 * @{
 */
/**
 * @brief  Reads the status of the pin assigned to the INT line of IRPyro
 *
 *         When the INT field in the AFE register is set to 1 the INT line on
 *         the IRPyro will go LOW when data is ready.
 * @param  port            IRPyro port number
 * @param  pin             IRPyro pin number to the INT pin
 * @retval INT_pin_status  INT pin is asserted LOW return (1)
 * @attention  Modify this function so the INT pin acts as an  INPUT
 */
uint8_t INT_pin_read(uint8_t port, uint8_t pin)
{
    GPIO_TypeDef* GPIOx;
    uint16_t GPIO_Pin = 1 << pin;
    uint8_t INT_pin_status = 0x00;
    switch (port)
    {
    case 1:
        GPIOx = GPIOA;
        break;
    case 2:
        GPIOx = GPIOB;
        break;
    default:
        GPIOx = GPIOA;
        break;
    }
    INT_pin_status = (0 == GPIO_ReadInputDataBit(GPIOx, GPIO_Pin)) ? 1 : 0;
    return INT_pin_status;
}
/**
 * @brief  ON/OFF control from the MCU to the IRPyro sensor
 *
 *         CS HIGH enables the IRPyro, CS LOW effectively turns OFF the sensor
 * @param  port  IRPyro port number
 * @param  pin   IRPyro pin number to the CS pin
 * @param  value 0/1 for OFF/ON
 * @retval None
 * @attention If this function is modified, the affected pins of the MCU should
 *            work as an OUTPUT
 */
void CS_pin_set(uint8_t port, uint8_t pin, uint8_t value)
{
    GPIO_TypeDef* GPIOx;
    uint16_t GPIO_Pin = 1<<pin;
    BitAction BitVal;
    switch (port)
    {
    case 1:
        GPIOx = GPIOA;
        break;
    case 2:
        GPIOx = GPIOB;
        break;
    default:
        GPIOx = GPIOA;
        break;
    }
    BitVal = (value == 0) ? Bit_RESET : Bit_SET;
    if ( BitVal )
    {
        GPIO_SetBits(GPIOx,GPIO_Pin) ;
    }
    else
    {
        GPIO_ResetBits(GPIOx,GPIO_Pin) ;
    }
}
/**
 * @brief  Power sequence from the MCU to the IRPyro sensor
 *
 * @param  port  IRPyro port number
 * @param  pin   IRPyro pin number to the CS pin
 * @retval None
 * @attention Refer to section "12.8 Device Power Up and Reset" in the Reference Manual
 */
void power_set(uint8_t port, uint8_t pin)
{
    CS_pin_set(port, pin, 0);
    CS_pin_set(port, pin, 1);
    Micro_controller_delay_us(50);
    CS_pin_set(port, pin, 0);
    Micro_controller_delay_us(2);
    CS_pin_set(port, pin, 1);
    Micro_controller_delay_us(2);
}
/** end of IRPyro GPIO control functions.
 * @}
 */
/** end of IRPyro access to General I/O functions (GPIO)
 * @}
 */
/* I2C Private Variables ----------------------------------------------------------*/
static I2C_Config_Type  Config  = {0};  ///< Configuration Structure for the I2C Module
static uint8_t Config_Valid_Flag =0;   ///< Configuration valid flag indicating module has been configured
/** @addtogroup IRPyro_I2C
 * @{
 */
/** @defgroup I2C_G3_IRPyro I2C Initialization Function
 * @brief    Initializes the I2C structure
 * @verbatim
 *    The I2C implementation on the STM32F3 micro controllers
 *    provides architecture independence to the IRPyro API
 * @endverbatim
 * @note Modify accordingly depending on architecture
 * @{
 */
/**
 * @brief  Initializes the I2C Hardware (I2C1).
 * @param[in]  External_Config Data structure with the I2C definition according to STM
 * @retval      None
 * @verbatim
 *            This function uses the ST hardware libraries to configure the
 *            registers responsible for the I2C hardware. This function sets
 *            the bus mode (master/slave), clock speed, master address, etc.
 *            This function also sets which pins should be used for the I2C bus.
 * @endverbatim
 * @attention Modify this function to allow the chosen MCU access to the I2C bus
 *            as a master
 */
void i2c_Init(I2C_Config_Type* External_Config)
{
    // Structure Declaration
    I2C_InitTypeDef I2C_InitStructure = {0};
    // Check for pointer
    if (External_Config)
    {
        // Set Configuration Structure
        Config.Timeout = External_Config->Timeout;
        Config.CLK_Timing = External_Config->CLK_Timing;
        // Enable Peripheral clock
        RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1, ENABLE);
        RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);
        // Set up alternate function pin config
        GPIO_PinAFConfig(External_Config->GPIOx, External_Config->SCL_Src, GPIO_AF_4);
        GPIO_PinAFConfig(External_Config->GPIOx, External_Config->SDA_Src, GPIO_AF_4);
        // Deinitialise I2C
        I2C_DeInit(I2C1);
        I2C_SoftwareResetCmd(I2C1);
        // Define I2C Init structure
        I2C_InitStructure.I2C_Timing = Config.CLK_Timing;              // Controls the CLK for I2C Bus
        I2C_InitStructure.I2C_AnalogFilter = I2C_AnalogFilter_Disable; // I2C_AnalogFilter_Enable;
        I2C_InitStructure.I2C_DigitalFilter = 0x00;                    // This parameter can be a number between 0x00 and 0x0F */
        I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;                     // Other options are MSBus.
        I2C_InitStructure.I2C_OwnAddress1 = 0x00;                      // Is there a standard for the master address?
        I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;                    // I guess this refers to the bit after every command
        I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;   //Not sure about this one.
        // Initialize structure
        I2C_Init(I2C1, &I2C_InitStructure);
        // Enable I2C Hardware
        SYSCFG_I2CFastModePlusConfig(SYSCFG_I2CFastModePlus_I2C1, ENABLE);
        I2C_Cmd(I2C1, ENABLE);
        // Set Config valid flag
        Config_Valid_Flag = 1;
    }
}
/** end of I2C Initialization Function
 * @}
 */
/** @defgroup I2C_G4_IRPyro I2C Control Functions
 * @brief    Master to slave read and write functions
 * @verbatim
 *    The I2C implementation on the STM32F3 micro controllers
 *    provides architecture independence to the IRPyro API
 * @endverbatim
 * @note Modify accordingly depending on architecture
 * @{
 */
/**
 * @brief  Sends data via I2C port.
 * @param[in] DeviceAddr  Slave device address
 * @param[in] Command  First byte to be sent
 * @param[in,out] *pBuffer: Remaining data bytes to be sent
 * @param[in] len Number of bytes to be sent
 * @retval     Pass/Fail
 * @verbatim
 *            This function sends an array of bytes over the I2C bus of a given
 *            length. The function is responsible for setting up the I2C transfer
 *            handling, i.e. calling the slave address and sending the data if an ACK is received.
 * @endverbatim
 * @attention Modify this function to implement an I2C write that is platform specific
 * @todo Use of GOTO on the I2C read and write functions  in order to detect what
 * causes the function to return. Needs to be addressed. Multiple returns reduce
 * complexity but is difficult to trace. This functions could be re-factored since
 * there are several waiting loops, required for the flags in use
 */
uint8_t i2c_Write(uint8_t DeviceAddr ,uint8_t Command, uint8_t* pBuffer, uint8_t len)
{
    // Local Variables
    uint32_t timeout = Config.Timeout;
    uint32_t counter = 0;
    uint8_t op_code = 0;
    // Exit if configuration is not valid
    if (!Config_Valid_Flag)
    {
        op_code = 0; // no configuration
        goto finish;
    }
    // Wait until I2C is not busy
    while (I2C_GetFlagStatus(I2C1, I2C_ISR_BUSY) != RESET)
    {
        if ((--timeout) == 0)
        {
            op_code = 2; // I2C_ISR_BUSY time out
            goto finish;
        }
    }
    // Flush buffer before transmitting
    I2C_Cmd(I2C1, DISABLE);
    I2C_Cmd(I2C1, ENABLE);
    // Configure slave address, nbytes, reload, end mode and start or stop generation //
    I2C_TransferHandling(I2C1, ((DeviceAddr& 0x7f) << 1),(len+1), I2C_AutoEnd_Mode, I2C_Generate_Start_Write);
    // Wait until TXIS flag is set //
    timeout = Config.Timeout;
    while (I2C_GetFlagStatus(I2C1, I2C_ISR_TXIS) == RESET)
    {
        if ((--timeout) == 0)
        {
            op_code = 3; // I2C_ISR_TXIS time out
            goto finish;
        }
    }
    // Send the set i2c Command
    I2C_SendData(I2C1, Command);
    // Now send the len amount of data bytes
    while (counter < len)
    {
        // Wait until TXIS flag is set //
        timeout = Config.Timeout;
        while (I2C_GetFlagStatus(I2C1, I2C_ISR_TXE) == RESET)
        {
            if ((--timeout) == 0)
            {
                op_code = 4; // I2C_ISR_TXE time out
                goto finish;
            }
        }
        // Send Data
        I2C_SendData(I2C1, pBuffer[counter]);
        ++counter;
    }
    // Wait until TC flag is set
    timeout = Config.Timeout;
    while (I2C_GetFlagStatus(I2C1, I2C_ISR_TXE) == RESET)
    {
        if ((--timeout) == 0)
        {
            op_code = 5; // I2C_ISR_TXE time out
            goto finish;
        }
    }
    // Clear STOPF flag
    I2C_ClearFlag(I2C1, I2C_ICR_STOPCF);
    op_code = 1; // success!
    // All operations completed OK return true
finish:
    // if(op_code != 1) op_code = 0; // values <> 1,0 are for debugging only
    return op_code;
}
/**
 * @brief  Reads data via I2C port.
 * @param   DeviceAddr: Slave device address
 * @param   Command: First byte to be sent
 * @param   *pBuffer: Pointer to receiving bytes array
 * @param   len: Number of bytes to be received
 * @retval Pass/Fail
 * @verbatim This function is used to receive an array of bytes over the I2C bus
 *           of a given length. The function is responsible for setting up the
 *           I2C transfer handling, i.e. calling the slave address and reading
 *           the data if an ACK is received.
 * @endverbatim
 * @attention Modify this function to implement an I2C read that is platform specific
 * @todo Use of GOTO on the I2C read and write functions  in order to detect what
 * causes the function to return. Needs to be addressed. Multiple returns reduce
 * complexity but is difficult to trace. This functions could be re-factored since
 * there are several waiting loops, required for the flags in use.
 */
uint8_t i2c_Read(uint8_t DeviceAddr,uint8_t Command, uint8_t* pBuffer, uint16_t len)
{
    // local time out counter
    uint32_t timeout = Config.Timeout;
    uint8_t op_code =0;
    // Exit if configuration is not valid
    if (!Config_Valid_Flag)
    {
        op_code = 0; // no configuration
        goto finish;
    }
    // Test on BUSY Flag //
    while (I2C_GetFlagStatus(I2C1, I2C_ISR_BUSY) != RESET)
    {
        if ((--timeout) == 0)
        {
            op_code = 2; // I2C_ISR_BUSY time out
            goto finish;
        }
    }
    // Configure slave address, nbytes, reload, end mode and start or stop generation //
    I2C_TransferHandling(I2C1, (DeviceAddr << 1), 1, I2C_SoftEnd_Mode, I2C_Generate_Start_Write);
    // Wait until TXIS flag is set
    timeout = Config.Timeout;
    while (I2C_GetFlagStatus(I2C1, I2C_ISR_TXE) == RESET)
    {
        if ((--timeout) == 0)
        {
            op_code = 3; // I2C_ISR_TXE time out
            goto finish;
        }
    }
    // Send ASIC command //
    I2C_SendData(I2C1, Command);
    // Wait until TC flag is set //
    timeout = Config.Timeout;
    while (I2C_GetFlagStatus(I2C1, I2C_ISR_TC) == RESET)
    {
        if ((--timeout) == 0)
        {
            op_code = 4; // I2C_ISR_TC time out
            goto finish;
        }
    }
    // Configure slave address, nbytes, reload, end mode and start or stop generation //
    I2C_TransferHandling(I2C1, (DeviceAddr << 1), len, I2C_AutoEnd_Mode, I2C_Generate_Start_Read);
    // Wait until all data are received //
    while (len)
    {
        // Wait until RXNE flag is set //
        timeout = Config.Timeout;
        while (I2C_GetFlagStatus(I2C1, I2C_ISR_RXNE) == RESET)
        {
            if ((--timeout) == 0)
            {
                op_code = 5; // I2C_ISR_RXNE time out
                goto finish;
            }
        }
        // Read data from RXDR //
        *pBuffer = I2C_ReceiveData(I2C1);
        // Point to the next location where the byte read will be saved //
        ++pBuffer;
        // Decrement the read bytes counter //
        --len;
    }
    // Wait until STOP flag is set //
    timeout = Config.Timeout;
    while (I2C_GetFlagStatus(I2C1, I2C_ISR_STOPF) == RESET)
    {
        if ((--timeout) == 0)
        {
            op_code = 6; // I2C_ISR_STOPF time out
            goto finish;
        }
    }
    /* Clear STOPF flag */
    I2C_ClearFlag(I2C1, I2C_ICR_STOPCF);
    op_code =1;
    /* All operations completed OK return true*/
finish:
    // if(op_code != 1) op_code = 0; // values <> 1,0 are for debugging only
    return op_code;
}
/** end of I2C Control Functions
 * @}
 */
/** end of IRPyro_I2C
 * @}
 */
/** @defgroup Micro_controller_interrupt_external External interrupt configuration
 * @brief Architecture dependent functions
 * @verbatim
 *  Functions to initialize ports, pins and interrupts handlers required to monitor
 *  external interrupts. This implementation is specific for the STM32 micro controllers
 *                 STM32F30x Peripherals Interrupt Handlers
 *  Add here the Interrupt Handler for the used peripheral(s) (PPP), for the
 *  available peripheral interrupt handler's name please refer to the startup
 *  file (startup_stm32f30x.s).
 * @endverbatim
 * @note Modify accordingly depending on architecture
 * @{
 */
/**
  * @brief  Configure Sensor 1 INT line in interrupt mode
  * @attention Initialization of GPIO is a prerequisite for successful configuration
  */
static void EXTI4_Config(void)
{
    EXTI_InitTypeDef   EXTI_InitStructure;
    NVIC_InitTypeDef   NVIC_InitStructure;
    /* Enable SYSCFG clock */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);
    /* Connect EXTI4 Line to PA4 pin */
    SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOA, EXTI_PinSource4);
    /* Enable and set EXTI15_10 Interrupt to the lowest priority */

    NVIC_InitStructure.NVIC_IRQChannel                   = EXTI4_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 3;    // EXTI4 priority group
    NVIC_InitStructure.NVIC_IRQChannelSubPriority        = 0;    // sub priority inside the group
    NVIC_InitStructure.NVIC_IRQChannelCmd                = ENABLE;
    NVIC_Init(&NVIC_InitStructure);
    NVIC_EnableIRQ(EXTI4_IRQn);

    /* Configure EXTI4 line */
    EXTI_InitStructure.EXTI_Line = EXTI_Line4;
    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init(&EXTI_InitStructure);
}
/**
  * @brief  Configure Sensor 2,3, and 4 INT line in interrupt mode
  * @attention Initialization of GPIO is a prerequisite for successful configuration
  */
static void EXTI9_5_Config(void)
{
    EXTI_InitTypeDef   EXTI_InitStructure;
    NVIC_InitTypeDef   NVIC_InitStructure;
    /* Enable SYSCFG clock */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);
    /* Connect EXTI5 Line to PA5 pin */
    SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOA, EXTI_PinSource5);
    /* Connect EXTI6 Line to PA6 pin */
    SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOA, EXTI_PinSource6);
    /* Connect EXTI7 Line to PA7 pin */
    SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOA, EXTI_PinSource7);
    /* Enable and set Button EXTI Interrupt to the lowest priority */

    NVIC_InitStructure.NVIC_IRQChannel                   = EXTI9_5_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 3;    // EXTI9_5 priority group
    NVIC_InitStructure.NVIC_IRQChannelSubPriority        = 1;    // sub priority inside the group
    NVIC_InitStructure.NVIC_IRQChannelCmd                = ENABLE;
    NVIC_Init(&NVIC_InitStructure);
    NVIC_EnableIRQ(EXTI9_5_IRQn);

    /* Configure Button EXTI line */
    EXTI_InitStructure.EXTI_Line    = EXTI_Line5;
    EXTI_InitStructure.EXTI_Mode    = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init(&EXTI_InitStructure);
    EXTI_InitStructure.EXTI_Line = EXTI_Line6;
    EXTI_Init(&EXTI_InitStructure);
    EXTI_InitStructure.EXTI_Line = EXTI_Line7;
    EXTI_Init(&EXTI_InitStructure);
}
/*  collects the status of the wake up event */
volatile uint8_t EXTI_line_record[4] = {0};
// typedef void (*EXTernal_callback)(void);
static EXTernal_callback EXTI_Callback_Fn = NULL;
/**
  * @brief   Starts function callbacks
  * @param   Callback     Function to call after each interval.
  */
static void EXTernal_Callback_Enable(EXTernal_callback Callback)
{
    EXTI_Callback_Fn        = Callback;
}
/**
  * @brief   Ends function callbacks.
  */
static void EXTernal_Callback_Disable(void)
{
    EXTI_Callback_Fn        = NULL;
}
/**
  * @brief   Configures  function callbacks
  * @param   Callback    Function to call after event detected
  */
void EXTernal_Callback_Setup(EXTernal_callback Callback)
{
    // Callback_Fn = Callback;
    if (Callback)
    {
        EXTernal_Callback_Enable(Callback);
    }
    else
    {
        EXTernal_Callback_Disable();
    }
}
/**
  * @brief  This function handles external line 4 interrupt request.
  */
void EXTI4_IRQHandler(void)
{
    if(EXTI_GetITStatus(EXTI_Line4) != RESET)
    {
        /* Signal Sensor 1 data is ready */
        EXTI_line_record[0]= 1;
        if (NULL != EXTI_Callback_Fn)
        {
            (*EXTI_Callback_Fn)();
            EXTI_line_record[0]= 0;
        }
        /* Clear the EXTI line 4 pending bit */
        EXTI_ClearITPendingBit(EXTI_Line4);
    }
}
/**
  * @brief  This function handles external lines 5 to 9 interrupt request.
  */
void EXTI9_5_IRQHandler(void)
{
    if((EXTI_GetITStatus(EXTI_Line7) != RESET))
    {
        /* Signal Sensor 4 data is ready */
        EXTI_line_record[3]= 1;
        if (NULL != EXTI_Callback_Fn)
        {
            (*EXTI_Callback_Fn)();
            EXTI_line_record[3]= 0;
        }
        /* Clear the EXTI line 7 pending bit */
        EXTI_ClearITPendingBit(EXTI_Line7);
    }
    if((EXTI_GetITStatus(EXTI_Line6) != RESET))
    {
        /* Signal Sensor 3 data is ready */
        EXTI_line_record[2]= 1;
        if (NULL != EXTI_Callback_Fn)
        {
            (*EXTI_Callback_Fn)();
            EXTI_line_record[2]= 0;
        }
        /* Clear the EXTI line 6 pending bit */
        EXTI_ClearITPendingBit(EXTI_Line6);
    }
    if((EXTI_GetITStatus(EXTI_Line5) != RESET))
    {
        /* Signal Sensor 2 data is ready */
        EXTI_line_record[1]= 1;
        if (NULL != EXTI_Callback_Fn)
        {
            (*EXTI_Callback_Fn)();
            EXTI_line_record[1]= 0;
        }
        /* Clear the EXTI line 5 pending bit */
        EXTI_ClearITPendingBit(EXTI_Line5);
    }
}
/**
  * @brief  This function access the status of the interrupt lines for the wake up event
  * @param  exti_line the exti_line to be questioned
  * @return 1 for exti_line awake 0 for no event detected
  */
uint8_t EXTI_detected(uint8_t exti_line)
{
    uint8_t op_result = 0;
    op_result = EXTI_line_record[exti_line];
    EXTI_line_record[exti_line] =0;
    return op_result;
}
/**
  * @} end of Micro_controller_interrupt_external group
  */
/**@defgroup USART_IRPyro IRPyro Functions access the USART
 * @brief Allows to get a byte from the USART and send a stream of data to a client
 * @note Modify accordingly depending on architecture
 * @{
 */
/* USART Private Variable-----------------------------------------------------*/
volatile uint8_t uart_byte; // char from UART
/* USART Private Functions ---------------------------------------------------*/
void UART_PutChar(USART_TypeDef* USARTx, uint8_t ch);
/**
  * @brief  Puts a char in the USART
  */
void UART_PutChar(USART_TypeDef* USARTx, uint8_t ch)
{
    while(!(USARTx->ISR & USART_ISR_TXE));
    USARTx->TDR = ch;
}
/**
  * @brief  Puts a string in the USART
  * @param USARTx usart
  * @param str
  */
void UART_PutString(USART_TypeDef* USARTx, uint8_t * str)
{
    while(*str != 0)
    {
        UART_PutChar(USARTx, *str);
        ++str;
    }
}
/**
  * @brief  Puts a number in the USART
  * @param USARTx usart port sending the number
  * @param number to be send
  */
void UART_PutNumber(USART_TypeDef* USARTx, uint32_t number)
{
    char value[10] = {0}; //a temp array to hold results of conversion
    int i = 0; //loop index
    do
    {
        value[i] = (char)(number % 10) + '0'; //convert integer to character
        ++i;
        number /= 10;
    }
    while(number);
    while(i) //send data
    {
        UART_PutChar(USARTx, value[--i]);
    }
}
// typedef void (*USART_callback)(void);
static USART_callback USARTCallback_Tx_Fn = NULL;
static USART_callback USARTCallback_Rx_Fn = NULL;
/**
  * @brief   Starts function callbacks
  * @param   Callback     Function to call after each interval.
  */
static void USART_Callback_Tx_Enable(USART_callback Callback)
{
    USARTCallback_Tx_Fn        = Callback;
}
/**
  * @brief   Ends function callbacks.
  */
static void USART_Callback_Tx_Disable(void)
{
    USARTCallback_Tx_Fn        = NULL;
}
/**
  * @brief   Configures  function callbacks
  * @param   Callback    Function to call after event detected
  */
void USART_Callback_Tx_Setup(USART_callback Callback)
{
    if (Callback)
    {
        USART_Callback_Tx_Enable(Callback);
    }
    else
    {
        USART_Callback_Tx_Disable();
    }
}
static void USART_Callback_Rx_Enable(USART_callback Callback)
{
    USARTCallback_Rx_Fn        = Callback;
}
/**
  * @brief   Ends function callbacks.
  */
static void USART_Callback_Rx_Disable(void)
{
    USARTCallback_Rx_Fn        = NULL;
}
/**
  * @brief   Configures  function callbacks
  * @param   Callback    Function to call after event detected
  */
void USART_Callback_Rx_Setup(USART_callback Callback)
{
    if (Callback)
    {
        USART_Callback_Rx_Enable(Callback);
    }
    else
    {
        USART_Callback_Rx_Disable();
    }
}
/**
  * @brief  Handles incomming char from the USART and stores it in global uart_byte
  */
void USART2_IRQHandler()
{
// Get data from UART
    if(USART_GetITStatus(USART2 , USART_IT_RXNE) != RESET)
    {
        if (NULL != USARTCallback_Rx_Fn)
        {
            (*USARTCallback_Rx_Fn)();
        }
    }
// Send data to the UART
    if(USART_GetITStatus(USART2 , USART_IT_TXE) != RESET)
    {
        if (NULL != USARTCallback_Tx_Fn)
        {
            (*USARTCallback_Tx_Fn)();
        }
    }
}
/**
  * @brief  Interface to acces uart_byte
  */
uint8_t UART_byte_get()
{
    return uart_byte;
}
/**
  * @brief  Formats output and sends to USART as a string
  * @param format constant format string as with printf
  * @param ... variable argument, list pf values.
  */
static void UART_display_write(char const * format, ...)
{
    va_list  args;
    char  buf[80] = "";
    va_start(args, format);
    vsprintf(buf, format, args); /* buf contains the formatted string */
    UART_PutString(USART2, (uint8_t*)buf);
    va_end(args);                /* Clean up. Do NOT omit */
}
/**
  * @brief Plain text data for the active channels of the sensor including, frame counter and saturation byte
  * @param channel_value      array of channel values
  * @param channel_saturation encoded saturation bits
  */
void UART_stream_char_send(uint32_t *channel_value, uint8_t channel_saturation)
{
    UART_display_write("FRAME #, %5u, CH, %5u, %5u, %5u, %5u, SAT, %2u \r\n",
                       channel_value[5],
                       channel_value[1],
                       channel_value[2],
                       channel_value[3],
                       channel_value[4],
                       channel_saturation);
}
/**
  * @brief Builds a binary data packet for the channels of the sensor, frame counter and saturation byte
  * @param DataS raw data from the sensor
  */
void UART_stream_binary_send(uint8_t *DataS)
{
#define PACKET_SIZE 19
#define SATURATION_MASK 0x80
#define SATURATION_FLAG(CH,POS) ((CH & SATURATION_MASK) >> (7 - POS))
    uint8_t data[PACKET_SIZE] = {0};
    uint8_t index_data = 3, index_DataS =0;
    data[0] = 0xAA ;                    // start of packet marker
    data[1] = 0x02 ;                    // data type
    data[2] = 0x0E ;                    // 14 bytes of data
    for(uint8_t channel=0; channel < 4; ++ channel)
    {
        data[index_data++] = DataS[index_DataS ++]& 0x7f;
        data[index_data++] = DataS[index_DataS ++];
        data[index_data++] = DataS[index_DataS ++];
    }
//fc
    data[index_data++] = DataS[index_DataS++];
    data[index_data++] = DataS[index_DataS++];
//saturation byte
    data[17]  = SATURATION_FLAG(DataS[0],0);
    data[17] += SATURATION_FLAG(DataS[3],1);
    data[17] += SATURATION_FLAG(DataS[6],2);
    data[17] += SATURATION_FLAG(DataS[9],3);
    data[18] = 0x55 ;                  // footer
    for(uint8_t index_packet=0; index_packet < PACKET_SIZE; ++index_packet)
    {
        UART_PutChar(USART2, data[index_packet]);    // send data
    }
}
/** end of IRPyro_USART
 * @}
 */
/** @defgroup EMITTER Status LEDs and emitter management
 *  @{
 */
/**
* @brief configures and enables status LED's on the backplane boards
*/
static void Micro_controller_LED_configure(void)
{
    // READY and RUNNING LEDs on the IRPyro Base Board (dIRBB01) and demo boards
    // set pins 0 and 1 of port A as normal OUTPUTS
    // Set pin 3 for ASIC_PS power control pin on the IRPB_027 rev B backplanes
    GPIO_structure_init(GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_3,
                        GPIO_Mode_OUT,
                        GPIO_OType_PP,
                        GPIO_Speed_2MHz,
                        GPIO_PuPd_NOPULL,
                        GPIOA );

    // Enable ASIC_PS pin for IRPB_027 boards (IRPyro TO)
    GPIO_SetBits(GPIOA, GPIO_Pin_3);
}
/**
* @brief Controls RUN led state
*/
static void LED_RUN(uint8_t LED_run)
{
#define LED_RUN_ON    GPIO_ResetBits(GPIOA,GPIO_Pin_1);
#define LED_RUN_OFF   GPIO_SetBits(GPIOA,GPIO_Pin_1);
    if (LED_run)
    {
        LED_RUN_ON
    }
    else
    {
        LED_RUN_OFF
    }
};
/**
* @brief Control for the READY led state
*/
void LED_READY(uint8_t LED_ready)
{
#define LED_READY_ON  GPIO_ResetBits(GPIOA,GPIO_Pin_0) ;
#define LED_READY_OFF GPIO_SetBits(GPIOA,GPIO_Pin_0) ;
    if (LED_ready)
    {
        LED_READY_ON
    }
    else
    {
        LED_READY_OFF
    }
};
/**
* @brief Configuration when the INTERNAL emitter is selected
*/
static void Emitter_configure_as_output(void)
{
    // Emitter on the IRPyro Base Board (dIRBB01) and demo boards
    // set pin 0 of port B as normal OUTPUT
    GPIO_structure_init(GPIO_Pin_0,
                        GPIO_Mode_OUT,
                        GPIO_OType_PP,
                        GPIO_Speed_2MHz,
                        GPIO_PuPd_NOPULL,
                        GPIOB );
};
/**
* @brief Configuration when the EXTERNAL emitter is selected
*/
static void Emitter_configure_as_input(void)
{
    GPIO_ResetBits(GPIOB,1) ;
    // Emitter on the IRPyro Base Board (dIRBB01) and demo boards
    // set pin 0 of port B as normal INPUT
    GPIO_structure_init(GPIO_Pin_0,
                        GPIO_Mode_IN,
                        GPIO_OType_PP,
                        GPIO_Speed_2MHz,
                        GPIO_PuPd_UP,
                        GPIOB );
};
static volatile uint16_t EmitterTimer;
/**
* @brief
*/
static void Emitter_status_and_timer_set(uint8_t *EmitterStatus, uint8_t EmitterONTime)
{
    if ( 2 == *EmitterStatus )
    {
        GPIO_SetBits(GPIOB,GPIO_Pin_0) ;
        EmitterTimer  = EmitterONTime ;
        *EmitterStatus = 0x06 ;    // internal emitter in use and ON
    }
    else
    {
        GPIO_ResetBits(GPIOB,GPIO_Pin_0) ;
        EmitterTimer  = 0 ;
        *EmitterStatus = 0x0A ;   // internal emitter in use and permanently OFF
    }
};
/**
* @brief
*/
static void Emitter_toggle(uint8_t *EmitterStatus, uint8_t EmitterONTime, uint8_t EmitterOFFTime)
{
    if ( 4 == ( *EmitterStatus & 0x04 ) )
    {
        GPIO_ResetBits(GPIOB, GPIO_Pin_0) ;
        EmitterTimer = EmitterOFFTime ;
    }
    else
    {
        GPIO_SetBits(GPIOB, GPIO_Pin_0) ;
        EmitterTimer = EmitterONTime ;
    }
}
/**
* @brief
*/
static void Emitter_status_and_timer_update(uint8_t *EmitterStatus, uint8_t EmitterONTime, uint8_t EmitterOFFTime)
{
    if ( ( 2 == ( *EmitterStatus & 0xA ) ) && ( 0 == EmitterTimer ) )
    {
        Emitter_toggle( EmitterStatus,  EmitterONTime,  EmitterOFFTime);
        *EmitterStatus ^= 0x04 ;
    }
};
/**
* @brief
*/
static uint8_t Emitter_status_evaluate(uint8_t EmitterStatus)
{
    if (EmitterStatus == 0 || EmitterStatus == 1)
    {
        // Internal Emitter is OFF or External Emitter is ON
        return 1;
    }
    else
    {
        // Internal Emitter is in use
        return 0;
    }
};
/**
* @brief Changes Emitter status and applies timing parameters
*/
void Emitter_control(uint8_t *EmitterStatus, uint8_t EmitterONTime, uint8_t EmitterOFFTime)
{
// handle emitter
    if ( 0x80 == ( 0x80 & *EmitterStatus ) )
    {
        /*  Emitter status has just changed
        If now 0, neither external nor internal function is enabled, so turn OFF output & set 'output ON' status to 0 but also set pin as input
        if now 1, external function is enabled, so ditto
        If now 2, internal function is enabled, so enable as output & turn ON
        If now 3, internal function is enabled but output permanently OFF
        */
        *EmitterStatus &= 0x7F ;
        if (Emitter_status_evaluate(*EmitterStatus))
        {
            Emitter_configure_as_input();
            *EmitterStatus &= 0xFB ; // clear 'internal emitter ON' flag
            EmitterTimer    = 0 ;
        }
        else
        {
            Emitter_configure_as_output();
            Emitter_status_and_timer_set(EmitterStatus, EmitterONTime);
        }
    }
    Emitter_status_and_timer_update(EmitterStatus, EmitterONTime, EmitterOFFTime);
}
/**
* @brief Configuration for hardware timer for emitter control
*/
static void Micro_controller_TIM_init(void)
{
    NVIC_InitTypeDef   NVIC_InitStructure ;

    NVIC_InitStructure.NVIC_IRQChannel                   = TIM3_IRQn;          // we want to configure the TIM3 update interrupts
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;    // TIM3 priority group
    NVIC_InitStructure.NVIC_IRQChannelSubPriority        = 0;    // sub priority inside the group
    NVIC_InitStructure.NVIC_IRQChannelCmd                = ENABLE ;            // the TIM3 interrupts are globally enabled
    NVIC_Init(&NVIC_InitStructure);                                            // the properties are passed to the NVIC_Init function which takes care of the low level stuff

    TIM_TimeBaseInitTypeDef     TIM_TimeBaseInitStruct ;

    //Timer Clock for RUNNING LED
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM3, ENABLE);
    //Disable Interrupt
    TIM_ITConfig(TIM3, TIM_IT_Update, DISABLE);
    //Define Timer Structure
    TIM_TimeBaseInitStruct.TIM_Prescaler               = 1;                  // Divides by 1
    TIM_TimeBaseInitStruct.TIM_CounterMode             = TIM_CounterMode_Up; // Count upwards
    TIM_TimeBaseInitStruct.TIM_Period                  = 30000;              // Divides by 30,000 making 1ms
    TIM_TimeBaseInitStruct.TIM_ClockDivision           = TIM_CKD_DIV1;       // Sets ratio between timer clock and sampling clock used in digital filters (not relevant here)
    //Initialise Timer Structure
    TIM_TimeBaseInit(TIM3, &TIM_TimeBaseInitStruct);
    //Enable Interrupt
    TIM_ITConfig(TIM3, TIM_IT_Update, ENABLE);
    //Enable interrupt Handler
    NVIC_EnableIRQ(TIM3_IRQn);
    //Enable timer
    TIM_Cmd(TIM3, ENABLE);
};
volatile uint8_t led_run_value   = 0;
volatile uint16_t tim3_counter   = 0;
volatile uint8_t led_ready_value = 0;
volatile uint32_t sample_period_timer = 0;
//typedef void (*TIM_callback)(void);
static TIM_callback TIM_Callback_Fn = NULL;
void Micro_controller_sample_period_timer_set(uint32_t count_value)
{
    sample_period_timer = count_value;
}
uint32_t Micro_controller_sample_period_timer_get(void)
{
    return sample_period_timer;
}
/**
  * @brief   Starts function callbacks
  * @param   Callback     Function to call after each interval.
  */
static void TIM_Callback_Enable(EXTernal_callback Callback)
{
    TIM_Callback_Fn        = Callback;
}
/**
  * @brief   Ends function callbacks.
  */
static void TIM_Callback_Disable(void)
{
    TIM_Callback_Fn        = NULL;
}
/**
  * @brief   Configures  function callbacks
  * @param   Callback    Function to call after event detected
  */
void TIM_Callback_Setup(TIM_callback Callback)
{
    if (Callback)
    {
        TIM_Callback_Enable(Callback);
    }
    else
    {
        TIM_Callback_Disable();
    }
}
/**
* @brief Interrupt timer for running LED on backplane boards
*/
void TIM3_IRQHandler (void)
{
    //Clear Timer Flag
    TIM_ClearITPendingBit(TIM3, TIM_IT_Update);
    if ( sample_period_timer > 0 )
    {
        --sample_period_timer ;
    }
    if ( EmitterTimer > 0 )
    {
        --EmitterTimer ;
    }
    if (tim3_counter > 500)
    {
        led_run_value = (!led_run_value) & 1;
        tim3_counter = 0;
        LED_RUN(led_run_value);
        if (NULL != TIM_Callback_Fn)
        {
            (*TIM_Callback_Fn)();
        }
    }
    else
    {
        ++ tim3_counter;
    }
//    led_ready_value = (!led_ready_value) & 1;
//    LED_READY(led_ready_value);
}
/** end of Status LEDs and emitter management
 * @}
 */
/** end of IRPyro_microcontroller_interface
 * @}
 */

/* ********** Copyright (c) 2018 _KEMET, Ltd. **********END OF FILE************/
